
#include <stdio.h>
#include <stdlib.h>
#include "tt_gps.h"
#include "tt_timer.h"
#include "tt_lcd.h"  

// AT Commands to get GPS GLL Info
char at_gps_sleep[] =("AT+CGPSPWR=0\r");
char at_gps_on[]    =("AT+CGPSPWR=1\r");
char at_gps_reset[] =("AT+CGPSRST=1\r");
char at_gps_status[]=("AT+CGPSSTATUS=?\r");
char at_gps_info4[] =("AT+CGPSINF=4\r");
char ok[]           =("\n\rOK\n\r");
char errr[]         =("Blah Blub\n");

// GPS Response Messages
char loc_unknown[]    =("\n\r+CGPSSTATUS: Location Unknown\r\n");
char loc_not_fixed[]  =("\n\r+CGPSSTATUS: Location Not Fix\r\n");
char loc_2D_fix[]     =("\n\r+CGPSSTATUS: Location 2D Fix\r\n");
char loc_3D_fix[]     =("\n\r+CGPSSTATUS: Location 3D Fix\r\n");

unsigned int gps_state;

void process_GPS_DATA(char *rx_buf)
{
	if (strcmp(rx_buf,"Ok\r\n")==0)
	{
		switch(gps_state)
		{
			case GPS_TURN_ON:
			case GPS_RESET:
			case GPS_STATUS:
			case GPS_RUNNING:
			case GPS_INF4:
				process_GPS_states();
			break;
		}
	}else if (strcmp(rx_buf,"Ok\r\n")!=0)
	{
		putstringLCD(rx_buf);
		increment_gps_timer_count();
		start_GPS_timer();
	}
}

 
void process_GPS_states()
{
	switch(gps_state)
        {
	        // Turn ON GPS
	        case GPS_INITIAL:	  
	        	gps_state = GPS_TURN_ON;
	        	init_gps_timer_count();      
	        	putstring_U2(at_gps_on);
	        	putstringLCD("Turned ON GPS\n");        	 
	       	break;
	       	
	       	// Turn ON GPS
	       	case GPS_TURN_ON:
	       		gps_state = GPS_RESET;
	       		putstring_U2(at_gps_reset);	 
	       		init_gps_timer_count();       		
	        	clrLCD();  
   				putstringLCD("RESET GPS\n");       	     
	       	break;
	       	
	       	// Turn OFF GPS
	       	case GPS_STATUS:
	       		gps_state = GPS_INF4;	        	
	        	putstring_U2(at_gps_info4); 
	       	break;
	       	
	       	// RESET GPS
	       	case GPS_RESET:
	        	// Start timer and wait for 50 Seconds
	        	gps_state = GPS_RUNNING;
	        	putstringLCD("WAITING...\n");
	        	start_GPS_timer();	        
	       	break;
	       	
	       	case GPS_INF4:
	        	// Get GPS Info
	        	gps_state = GPS_SLEEP;
	        	putstringLCD("GPS Info RX\n");	        	
	        	putstring_U2(at_gps_sleep);	        
	       	break;
	       	
	       	case GPS_RUNNING:
	        // Check if the timer is still running
	        	gps_state = GPS_STATUS;
	        	putstring_U2(at_gps_status);
	        // If time out happened then exit loop	        
	       	break;
	       	
	       	case GPS_TIMEOUT:
	        	// No Sattelite in view, turn OFF GPS Module
	        	gps_state = GPS_SLEEP;
	        	putstringLCD("No Sattelite in view\n");
	        	putstring_U2(at_gps_sleep);	          	      
	       	break;
	       	
	       	case GPS_SLEEP:
	       		// Tag alert received - turn ON GPS	
	       		clrLCD();  
   				putstringLCD("Tag RX Triggered");   				gps_state = GPS_TURN_ON;	        	
	        	putstring_U2(at_gps_on);
	       	break; 
	       	default:
	       		putstringLCD("Unknown State");
	       	break;        
        }// Switch - case       
}// process_GPS_States

