
#include <p24fj128ga010.h>
#include "tt_generic.h"
#include "tt_ring.h"
#include "tt_timer.h"
#include "tt_lcd.h"
#include "tt_sms.h"
#include "tt_gprs.h"

typedef enum  {
  STATE_WAIT_FOR_FALL_EDGE = 0,
  STATE_WAIT_FOR_RISE_EDGE,
} ICSTATE;

ICSTATE e_isrICState = STATE_WAIT_FOR_FALL_EDGE;

volatile uint16_t u16_oflowCount = 0;
volatile uint8_t u8_captureFlag = 0;
volatile uint16_t u16_lastCapture;
volatile uint16_t u16_thisCapture;
volatile uint32_t u32_pulseWidth;


void configInputCapture2(void)
{
	CONFIG_RINGI();
	IC2CON = IC_TIMER3_SRC |   //Timer 3 source
           IC_INT_1CAPTURE |   //Interrupt every capture
           IC_IDLE_CON     |   // IC operate in sleep mode 
           IC_EVERY_EDGE;      //Capture every edge
           
  _IC2IF = 0;
  _IC2IP = 2;   //higher than Timer3 so that Timer3 does not interrupt IC2
  _IC2IE = 1;   //enable
}


//void _ISRFAST _IC2Interrupt() 
void __attribute__((__interrupt__, auto_psv)) _IC2Interrupt(void)
{

  _IC2IF = 0;
  u16_thisCapture = IC2BUF;  //always read the buffer to prevent overflow
  clrLCD();
  putstringLCD("SMS RXed");
  main_mode = MODE_SMS;
  //process_SMS_states();
  
  
  switch (e_isrICState) {
    case STATE_WAIT_FOR_FALL_EDGE:
    clrLCD();
    putstringLCD("SMS RXed");
    
      if (u8_captureFlag == 0) {
        if (u16_thisCapture == 0 && _T3IF) u16_oflowCount = 0 -1; //simultaneous timer with capture
        else u16_oflowCount = 0;
        u16_lastCapture = u16_thisCapture;
        e_isrICState = STATE_WAIT_FOR_RISE_EDGE;
      }
      break;
    case STATE_WAIT_FOR_RISE_EDGE:
    putstringLCD("RI HIGH");
      if (u16_thisCapture == 0 && _T3IF) u16_oflowCount++; //simultaneous interrupt, increment oflow count
      u32_pulseWidth = computeDeltaTicksLong(u16_lastCapture,u16_thisCapture,PR3, u16_oflowCount); //get delta ticks
      u32_pulseWidth = ticksToUs(u32_pulseWidth,getTimerPrescale(T3CONbits)); //convert to microseconds
      u8_captureFlag = 1;
      e_isrICState = STATE_WAIT_FOR_FALL_EDGE;
      break;
    default:
      e_isrICState = STATE_WAIT_FOR_FALL_EDGE;
  }
  
}

