#include <stdio.h>
#include <stdlib.h> 
#include "tt_timer.h"
#include "tt_lcd.h" 
#include "tt_gps.h"
#include "tt_ring.h"

#define GPS_WAIT_PERIOD     300      // in ms

unsigned int gps_timer_count;
unsigned int uart_timeout;



void clear_UART_timeout()
{
	stop_UART_timer();
	uart_timeout = 0;
}

unsigned int is_UART_timeout()
{
	return uart_timeout;
}

void set_UART_timeout(unsigned int timeout)
{
	uart_timeout = timeout;
}

void init_gps_timer_count()
{
	gps_timer_count = 0;
}


void increment_gps_timer_count()
{
	gps_timer_count++;
}

void init_GPS_timer()
{
	TMR2  = 0;           //clear timer2 value  				
	T2CON = T2_OFF | T2_IDLE_CON | T2_GATE_OFF | T2_32BIT_MODE_OFF | T2_SOURCE_INT | T2_PS_1_256 ; // Results in T2CON = 0x0020
	PR2 = 0xF424;  //  450 Milli seconds 450/1000 * (8000000/256) -1 = 14062 = 0x36EE	
  	IPC1bits.T2IP = 6;	 //set interrupt priority 
  	start_GPS_timer();
} 

// Timer 2 is Used for GPS
void process_GPS_timer()
{	
	// switch case for gps states
	// putstringLCD("Waiting...\n");
	process_GPS_states();
}

// Timer 2 is Used for GPS
void start_GPS_timer()
{
	// Clear the Interrupt flag
	IFS0bits.T2IF = 0;
	//Enable the interrupt
	IEC0bits.T2IE = 1;
	// Turn On the Timer
	T2CONbits.TON = 1;
}

// Timer 2 is Used for GPS
void stop_GPS_timer()
{
	// Turn OFF the Timer
	T2CONbits.TON = 0;
	// Disable the interrupt
	IEC0bits.T2IE = 0;
	// Clear the Interrupt flag
	IFS0bits.T2IF = 0;		
}

void init_UART_timer()
{
	TMR3  = 0;           //clear timer3 value  				
	T3CON = T3_OFF | T3_IDLE_CON | T3_GATE_OFF | T3_32BIT_MODE_OFF | T3_SOURCE_INT | T3_PS_1_256 ; // Results in T2CON = 0x0020
	PR3 = 0x36EE;  //  450 Milli seconds 450/1000 * (8000000/256) -1 = 14062 = 0x36EE	
  	IPC2bits.T3IP = 7;	 //set interrupt priority   	
}

// Timer 3 is Used for UART
void start_UART_timer()
{
	// Clear the Interrupt flag
	IFS0bits.T3IF = 0;
	//Enable the interrupt
	IEC0bits.T3IE = 1;
	// Turn On the Timer
	T3CONbits.TON = 1;
}

// Timer 3 is Used for UART
void stop_UART_timer()
{
	// Turn OFF the Timer
	T3CONbits.TON = 0;
	// Disable the interrupt
	IEC0bits.T3IE = 0;
	// Clear the Interrupt flag
	IFS0bits.T3IF = 0;		
}

//  TODO GPRS Associated timer functions
void init_GPRS_timer(){}
void start_GPRS_timer(){}
void stop_GPRS_timer(){}
void process_GPRS_timer(){}

//  Ring Indicator Associated timer functions
void init_RI_timer()
{
	TMR3  = 0;           //clear timer3 value  				
	T3CON = T3_OFF | T3_IDLE_CON | T3_GATE_OFF | T3_32BIT_MODE_OFF | T3_SOURCE_INT | T3_PS_1_256 ; // Results in T2CON = 0x0020
	PR3 = 0xFFFF;  //  Maximum Speed
	_T3IF = 0; 
	_T3IE = 1;				
  	IPC2bits.T3IP = 1;	 //set interrupt priority   	
}

void start_RI_timer()
{
	// Clear the Interrupt flag
	IFS0bits.T3IF = 0;
	//Enable the interrupt
	IEC0bits.T3IE = 1;
	// Turn On the Timer
	T3CONbits.TON = 1;
}
void stop_RI_timer()
{
	// Turn OFF the Timer
	T3CONbits.TON = 0;
	// Disable the interrupt
	IEC0bits.T3IE = 0;
	// Clear the Interrupt flag
	IFS0bits.T3IF = 0;	
}

void process_RI_timer(){}

/** Given the TCKPS bitfield, return the timer prescale encoded
 *  by these bits. Use \ref getTimerPrescale as a convenient
 *  way to extract the TCKPS bitfield from a TxCONbits SFT
 *  then call this function.
 *  \param u8_TCKPS TCKPS bitfield from the timer in question
 *  \return Prescale value.
 */
uint16_t getTimerPrescaleBits(uint8_t u8_TCKPS) {
  uint16_t au16_prescaleValue[] = { 1, 8, 64, 256 };
  ASSERT(u8_TCKPS <= 3);
  return au16_prescaleValue[u8_TCKPS];
}


/** Converts timer ticks to microseconds
 *  \param u32_ticks  Timer ticks
 *  \param u16_tmrPre Timer prescale value
 *  \return time in microseconds
 */
uint32_t ticksToUs (uint32_t u32_ticks, uint16_t u16_tmrPre) {
  //because of the wide range of the numbers, use a float for precision
  float f_ticks;
  uint32_t u32_timeUs;

  f_ticks = u32_ticks;   //convert to float
  f_ticks = ((f_ticks * u16_tmrPre)/FCY) * 1000000L;
  u32_timeUs = roundFloatToUint32(f_ticks);  //back to int32_t
  return (u32_timeUs);
}



/** Computes delta ticks between two Timer register captures
 * Assumes long time interval and thus has a parameter for tracking timer overflows
 *  \param u16_start  start tick
 *  \param u16_end    end tick
 *  \param u16_tmrPR  Timer period register
 *  \param u16_oflows number of timer overflows
 *  \return delta ticks
 */
uint32_t computeDeltaTicksLong(uint16_t u16_start, uint16_t u16_end, uint16_t u16_tmrPR, uint16_t u16_oflows) {
  uint32_t u32_deltaTicks;
  if (u16_oflows == 0) u32_deltaTicks = u16_end - u16_start;
  else {
    //compute ticks from start to timer overflow
    u32_deltaTicks = (u16_tmrPR + 1) - u16_start;
    //add ticks due to overflows =  (overflows -1) * ticks_per_overflow
    u32_deltaTicks += ((((uint32_t) u16_oflows)- 1) * (((uint32_t)u16_tmrPR) + 1)) ;
    //now add in the delta due to the last capture
    u32_deltaTicks += u16_end;
  }
  return (u32_deltaTicks);
}



void __attribute__((__interrupt__, auto_psv)) _T2Interrupt(void)
{   
	// Stop the timer
	// clear the timer flags	
   stop_GPS_timer();
   // check if max timer count is reached
   if(gps_timer_count <= 15)
   {
	   increment_gps_timer_count();
	   start_GPS_timer();
		//process_GPS_timer();   		
   } 
   else
   {
	   //process_GPS_timer(); 
	   // display Satellite is not reachable
	  // putstringLCD("No Satellites in Range\n");
   }
}

void __attribute__((__interrupt__, auto_psv)) _T3Interrupt(void)
{   
	// Stop the timer
	u16_oflowCount = 0;
	// clear the timer flags	
   stop_UART_timer();
   // time out
   set_UART_timeout(1);   
}
