
#include <stdint.h>
#include "tt_generic.h"


/* T1CON: TIMER1 CONTROL REGISTER */
#define T1_ON               0x8000 /* Timer1 ON */
#define T1_OFF              0x0000 /* Timer1 OFF */
#define T1_OFF_ON_MASK      (~T1_ON)

#define T1_IDLE_STOP        0x2000 /* stop operation during sleep */
#define T1_IDLE_CON         0x0000 /* operate during sleep */
#define T1_IDLE_MASK        (~T1_IDLE_STOP)

#define T1_GATE_ON          0x0040 /* Timer Gate time accumulation enabled */
#define T1_GATE_OFF         0x0000 /* Timer Gate time accumulation disabled */
#define T1_GATE_MASK        (~T1_GATE_ON)

#define T1_PS_1_1           0x0000 /* Prescaler 1:1 */
#define T1_PS_1_8           0x0010 /*           1:8 */
#define T1_PS_1_64          0x0020 /*          1:64 */
#define T1_PS_1_256         0x0030 /*         1:256 */
#define T1_PS_MASK          (~T1_PS_1_256)

#define T1_SYNC_EXT_ON      0x0004 /* Synch external clk input */
#define T1_SYNC_EXT_OFF     0x0000 /* Do not synch external clk input */
#define T1_SYNC_EXT_MASK    (~T1_SYNC_EXT_ON)

#define T1_SOURCE_EXT       0x0002 /* External clock source */
#define T1_SOURCE_INT       0x0000 /* Internal clock source */
#define T1_SOURCE_MASK      (~T1_SOURCE_EXT)


/* T2CON: TIMER2 CONTROL REGISTER */
#define T2_ON               0x8000      /* Timer2 ON */
#define T2_OFF              0x0000      /* Timer2 OFF */
#define T2_OFF_ON_MASK      (~T2_ON)

#define T2_IDLE_STOP        0x2000 /* stop operation during sleep */
#define T2_IDLE_CON         0x0000 /* operate during sleep */
#define T2_IDLE_MASK        (~T2_IDLE_STOP)

#define T2_GATE_ON          0x0040 /* Timer Gate time accumulation enabled */
#define T2_GATE_OFF         0x0000 /* Timer Gate time accumulation disabled */
#define T2_GATE_MASK        (~T2_GATE_ON)

#define T2_PS_1_1           0x0000      /* Prescaler 1:1 */
#define T2_PS_1_8           0x0010      /*           1:8 */
#define T2_PS_1_64          0x0020      /*          1:64 */
#define T2_PS_1_256         0x0030      /*         1:256 */
#define T2_PS_MASK          (~T2_PS_1_256)

#define T2_32BIT_MODE_ON    0x0008      /* Timer 2 and Timer 3 form a 32 bit Timer */
#define T2_32BIT_MODE_OFF   0x0000
#define T2_32BIT_MODE_MASK   (~T2_32BIT_MODE_ON)

#define T2_SOURCE_EXT       0x0002 /* External clock source */
#define T2_SOURCE_INT       0x0000 /* Internal clock source */
#define T2_SOURCE_MASK      (~T2_SOURCE_EXT)

/* T3CON: TIMER3 CONTROL REGISTER */
#define T3_ON               0x8000      /* Timer2 ON */
#define T3_OFF              0x0000      /* Timer2 OFF */
#define T3_OFF_ON_MASK      (~T3_ON)

#define T3_IDLE_STOP        0x2000 /* stop operation during sleep */
#define T3_IDLE_CON         0x0000 /* operate during sleep */
#define T3_IDLE_MASK        (~T3_IDLE_STOP)

#define T3_GATE_ON          0x0040 /* Timer Gate time accumulation enabled */
#define T3_GATE_OFF         0x0000 /* Timer Gate time accumulation disabled */
#define T3_GATE_MASK        (~T3_GATE_ON)

#define T3_PS_1_1           0x0000      /* Prescaler 1:1 */
#define T3_PS_1_8           0x0010      /*           1:8 */
#define T3_PS_1_64          0x0020      /*          1:64 */
#define T3_PS_1_256         0x0030      /*         1:256 */
#define T3_PS_MASK          (~T3_PS_1_256)

#define T3_32BIT_MODE_ON    0x0008      /* Timer 2 and Timer 3 form a 32 bit Timer */
#define T3_32BIT_MODE_OFF   0x0000
#define T3_32BIT_MODE_MASK   (~T3_32BIT_MODE_ON)

#define T3_SOURCE_EXT       0x0002 /* External clock source */
#define T3_SOURCE_INT       0x0000 /* Internal clock source */
#define T3_SOURCE_MASK      (~T3_SOURCE_EXT)


/* T4CON: TIMER4 CONTROL REGISTER */
#define T4_ON               0x8000      /* Timer4 ON */
#define T4_OFF              0x0000      /* Timer4 OFF */
#define T4_OFF_ON_MASK      (~T4_ON)

#define T4_IDLE_STOP        0x2000 /* stop operation during sleep */
#define T4_IDLE_CON         0x0000 /* operate during sleep */
#define T4_IDLE_MASK        (~T4_IDLE_STOP)

#define T4_GATE_ON          0x0040 /* Timer Gate time accumulation enabled */
#define T4_GATE_OFF         0x0000 /* Timer Gate time accumulation disabled */
#define T4_GATE_MASK        (~T4_GATE_ON)

#define T4_PS_1_1           0x0000      /* Prescaler 1:1 */
#define T4_PS_1_8           0x0010      /*           1:8 */
#define T4_PS_1_64          0x0020      /*          1:64 */
#define T4_PS_1_256         0x0030      /*         1:256 */
#define T4_PS_MASK          (~T4_PS_1_256)

#define T4_32BIT_MODE_ON    0x0008      /* Timer 4 and Timer 5 form a 32 bit Timer */
#define T4_32BIT_MODE_OFF   0x0000
#define T4_32BIT_MODE_MASK   (~T4_32BIT_MODE_ON)

#define T4_SOURCE_EXT       0x0002 /* External clock source */
#define T4_SOURCE_INT       0x0000 /* Internal clock source */
#define T4_SOURCE_MASK      (~T4_SOURCE_EXT)

#define IC_TIMER3_SRC       0x0000 /* Timer3 is the clock source for Capture */
#define IC_INT_1CAPTURE     0x0000 /* Interrupt on first Capture */

#define IC_EVERY_RISE_EDGE  0x0003 /* Every rising edge      */
#define IC_EVERY_FALL_EDGE  0x0002 /* Every falling edge      */
#define IC_EVERY_EDGE       0x0001 /* Every rising/falling edge */
#define IC_INPUTCAP_OFF     0x0000 /* Input Capture Off      */

#define IC_IDLE_CON         0x0000 /* IC operate in sleep mode */

/** Given the TxCON register as a bitfield, determines the prescale
 *  based on the TCKPS bitfield. Works for PIC24H, PIC24F timers.
 *  Simply accesses the TCKPS bitfield then calls
 *  \ref getTimerPrescaleBits with the bitfield.
 *  \param TxCONbits The TxCONbits for the timer in question
 *  \return Prescale value.
 */
#define getTimerPrescale(TxCONbits) getTimerPrescaleBits(TxCONbits.TCKPS)


extern unsigned int gps_timer_count;
extern unsigned int uart_timeout;

// Function prototypes
//  GPS Associated timer functions
void init_GPS_timer();
void start_GPS_timer();
void stop_GPS_timer();
void process_GPS_timer();

//  GPRS Associated timer functions
void init_GPRS_timer();
void start_GPRS_timer();
void stop_GPRS_timer();
void process_GPRS_timer();

// UART Associated timer functions
void init_UART_timer();
void clear_UART_timeout();
unsigned int is_UART_timeout();
void set_UART_timeout(unsigned int timeout);
void start_UART_timer();
void stop_UART_timer();

uint32_t computeDeltaTicksLong(uint16_t u16_start, uint16_t u16_end, uint16_t u16_tmrPR, uint16_t u16_oflows);


uint32_t ticksToUs (uint32_t u32_ticks, uint16_t u16_tmrPre);


//  Ring Indicator Associated timer functions
void init_RI_timer();
void start_RI_timer();
void stop_RI_timer();
void process_RI_timer();
