#include "buffer.h"

// I/O definitions for the Explorer16
#define U1_CTS     _RF12             // Cleart To Send, input, HW handshake
#define U1_RTS     _RF13             // Request To Send, output, HW handshake
#define U1_TRTS    TRISFbits.TRISF13 // tris control for RTS pin

#define U2_CTS     _RF12             // Cleart To Send, input, HW handshake
#define U2_RTS     _RF13             // Request To Send, output, HW handshake
#define U2_TRTS    TRISFbits.TRISF13 // tris control for RTS pin


// Pic baud rate calculator:
// FYc = 64 MHz @19200 gives 103 & 416 (BREG = 1)

// Also works for 4MHz @ 12 for sim908 but not for id-12

// timing and baud rate calculations - best match
// NEVER EVER change this -
#define U1_BRATE    416         // 19200 baud (BREGH=1)
#define U2_BRATE    51          // 19200 baud (BREGH=0)


/** Determine if a character is available in the UART's
 *  receive buffer.
 *  \return True (1) if character is available, 0 if not.
 */
#define IS_CHAR_READY_UART1() U1STAbits.URXDA
#define IS_CHAR_READY_UART2() U2STAbits.URXDA
//#define IS_CHAR_READY_UART3() U3STAbits.URXDA

/** Determine if a the transmit buffer is full.
 *  \return True (1) if the transmit buffer if full,
 *          false (0) if not.
 */
#define IS_TRANSMIT_BUFFER_FULL_UART1() U1STAbits.UTXBF
#define IS_TRANSMIT_BUFFER_FULL_UART2() U2STAbits.UTXBF
//#define IS_TRANSMIT_BUFFER_FULL_UART3() U3STAbits.UTXBF

/** Determines if all characters placed in the UART have been sent.
 *  Returns 1 if the last transmission has completed, or 0 if a transmission
 *  is in progress or queued in the transmit buffer.
 *  \return True (1) if the last transmission has completed, 0 if not.
 */
#define IS_TRANSMIT_COMPLETE_UART1() U1STAbits.TRMT
#define IS_TRANSMIT_COMPLETE_UART2() U2STAbits.TRMT
//#define IS_TRANSMIT_COMPLETE_UART3() U3STAbits.TRMT

/** Waits until all characters placed in the UART have been sent. */
inline static void WAIT_UNTIL_TRANSMIT_COMPLETE_UART1() {
  while (!IS_TRANSMIT_COMPLETE_UART1());;
    //doHeartbeat();
}

/** Waits until all characters placed in the UART have been sent. */
inline static void WAIT_UNTIL_TRANSMIT_COMPLETE_UART2() {
  while (!IS_TRANSMIT_COMPLETE_UART2());;
    //doHeartbeat();
}

/** Waits until all characters placed in the UART have been sent. */
/*
inline static void WAIT_UNTIL_TRANSMIT_COMPLETE_UART2() {
  while (!IS_TRANSMIT_COMPLETE_UART2())
    //doHeartbeat();
}*/

// init the serial port UART 1 & UART 2
// (UART1, 57200@32MHz, 8, N, 1, NONE )
// (UART2, 9600@32MHz, 8, N, 1, CTS/RTS )
// (UART3, 19200@32MHz, 8, N, 1, NONE )
void initUARTs( void); 
void initUARTInterrupts (void);

// UART Where the ID-12 is terminated
void init_U1( void);
// send a character to the serial port
int putChar_U1( int c);
// wait for a new character to arrive to the serial port
char getChar_U1(void); 
// send a null terminated string to the serial port
void putstring_U1(Buffer * buf);
// receive a null terminated string in a buffer of len char
//void getstring_U1( Buffer * u1b );
char * getstring_U1( char *u1s, int len);
//check for data in UART
int isCharReady_U1(void);
//check for UART RX error
void checkRxError_U1(void); 

// UAR Where the SIM908 GPS Port is terminated
void init_U2( void);
// send a character to the serial port
int putChar_U2( int c);
// wait for a new character to arrive to the serial port
char getChar_U2(void); 
// send a null terminated string to the serial port
void putstring_U2(Buffer * buf);
// receive a null terminated string in a buffer of len char
//void getstring_U2( Buffer * u2b );
char * getstring_U2( char *u2s, int len);
//check for data in UART
int isCharReady_U2(void);
//check for UART RX error
void checkRxError_U2(void);

//extern Buffer U1_RX_BUF; 
//extern Buffer U2_RX_BUF;

extern char U1_RX_BUF[20];
extern char U2_RX_BUF[180];
